@@ -25,6 +25,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <string.h>
+#include <limits.h>
 
 #include <glib/glist.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -74,7 +75,7 @@ static gboolean initialised = FALSE;
 static GtkWidget *progress_bar, *status_bar;
 
 /** The widgets in the main UI */
-static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *genre_entry, *track_listview;
+static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *genre_entry, *year_entry, *track_listview;
 
 /** The menuitem in the main menu */
 static GtkWidget *extract_menuitem, *play_menuitem, *reread_menuitem, *select_all_menuitem, *deselect_all_menuitem;
@@ -116,15 +117,18 @@ static guint cookie;
 /**
  * Build the absolute filename for the specified track.
  *
- * The base path is the extern variable 'base_path', the format to use
- * is the extern variable 'file_pattern'. Free the result when you
+ * The base path is the extern variable 'base_uri', the formats to use are the
+ * extern variables 'path_pattern' and 'file_pattern'. Free the result when you
  * have finished with it.
  */
 static char*
-build_filename (const TrackDetails *track)
+build_filename (const TrackDetails *track, GError **error)
 {
-  GnomeVFSURI *uri, *new;
-  char *realfile, *realpath, *filename, *string;
+  GnomeVFSURI *uri, *new; 
+  gchar *realfile, *realpath, *filename, *string;
+  const gchar *extension;
+  size_t len_extension;
+  int max_realfile = INT_MAX;
   GMAudioProfile *profile;
 
   g_object_get (extractor, "profile", &profile, NULL);
@@ -136,8 +140,23 @@ build_filename (const TrackDetails *track)
   gnome_vfs_uri_unref (uri); uri = new;
   g_free (realpath);
 
+  extension = gm_audio_profile_get_extension (profile);
+  len_extension = 1 + strlen (extension);
+#if defined(NAME_MAX) && NAME_MAX > 0
+  max_realfile = NAME_MAX - len_extension;
+#endif /* NAME_MAX */
+#if defined(PATH_MAX) && PATH_MAX > 0
+  if (!strcmp (gnome_vfs_uri_get_scheme (uri), "file")) {
+    size_t len_path = strlen (gnome_vfs_uri_get_path (uri)) + 1;
+    max_realfile = MIN (max_realfile, PATH_MAX - len_path - len_extension);
+  }
+#endif /* PATH_MAX */
+  if (max_realfile <= 0) {
+    g_set_error (error, SJ_ERROR, SJ_ERROR_INTERNAL_ERROR, g_strdup (gnome_vfs_result_to_string (GNOME_VFS_ERROR_NAME_TOO_LONG)));
+    return NULL;
+  }
   realfile = filepath_parse_pattern (file_pattern, track);
-  filename = g_strdup_printf("%s.%s", realfile, gm_audio_profile_get_extension(profile));
+  filename = g_strdup_printf("%.*s.%s", max_realfile, realfile, extension);
   new = gnome_vfs_uri_append_file_name (uri, filename);
   gnome_vfs_uri_unref (uri); uri = new;
   g_free (filename); g_free (realfile);
@@ -197,11 +216,13 @@ cleanup (void)
   gtk_statusbar_push (GTK_STATUSBAR (status_bar), 0, "");
   /* Clear the progress bar */
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0);
+  gtk_widget_hide (progress_bar);
   
   gtk_widget_set_sensitive (play_button, TRUE);
   gtk_widget_set_sensitive (title_entry, TRUE);
   gtk_widget_set_sensitive (artist_entry, TRUE);
   gtk_widget_set_sensitive (genre_entry, TRUE);
+  gtk_widget_set_sensitive (year_entry, TRUE);
   /* Enabling the Menuitem */ 
   gtk_widget_set_sensitive (play_menuitem, TRUE);
   gtk_widget_set_sensitive (extract_menuitem, TRUE);
@@ -314,7 +335,9 @@ create_directory_for (const char* url, GError **error)
 
   res = make_directory_with_parents_for_uri (parent, 0777);
   if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_FILE_EXISTS) {
-    g_set_error (error, SJ_ERROR, SJ_ERROR_CD_PERMISSION_ERROR, g_strdup (gnome_vfs_result_to_string (res)));
+    g_set_error (error, SJ_ERROR, SJ_ERROR_CD_PERMISSION_ERROR,
+                 _("Failed to create output directory: %s"),
+                 gnome_vfs_result_to_string (res));
     return NULL;
   }
 
@@ -323,8 +346,9 @@ create_directory_for (const char* url, GError **error)
   return string;
 }
 
-/* Prototype for pop_and_extract */
+/* Prototypes for pop_and_extract */
 static void on_completion_cb (SjExtractor *extractor, gpointer data);
+static void on_error_cb (SjExtractor *extractor, GError *error, gpointer data);
 
 /**
  * The work horse of this file.  Take the first entry from the pending list,
@@ -338,30 +362,20 @@ pop_and_extract (int *overwrite_mode)
     g_assert_not_reached ();
   } else {
     TrackDetails *track = NULL;
-    char *file_path, *directory, *text;
+    char *file_path = NULL, *directory;
     GError *error = NULL;
-    
+
     /* Pop the next track to extract */
     gtk_tree_model_get (GTK_TREE_MODEL (track_store), &current, COLUMN_DETAILS, &track, -1);
     /* Build the filename for this track */
-    file_path = build_filename (track);
+    file_path = build_filename (track, &error);
+    if (error) {
+      goto error;
+    }
     /* And create the directory it lives in */
     directory = create_directory_for (file_path, &error);
     if (error) {
-      GtkWidget *dialog;
-      text = g_strdup_printf (_("Sound Juicer could not extract this CD.\nReason: %s"), error->message);
-
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), 0,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       text);
-      g_free (text);
-
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      g_error_free (error);
-      cleanup ();
-      return;
+      goto error;
     }
     /* Save the directory name for later */
     paths = g_list_append (paths, directory);
@@ -376,7 +390,7 @@ pop_and_extract (int *overwrite_mode)
         ((file_size > MIN_FILE_SIZE) && (*overwrite_mode == SKIP_ALL))) {
       on_completion_cb (NULL, overwrite_mode);
       return;
-      }
+    }
 
     /* What if the file already exists? */
     if ((file_size > MIN_FILE_SIZE) &&
@@ -397,25 +411,21 @@ pop_and_extract (int *overwrite_mode)
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
                                    CLAMP ((float)current_duration / (float)total_duration, 0.0, 1.0));
 
+    /* Set the Treelist focus to the item to be extracted */
+    GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL (track_store), &current); 
+    gtk_tree_view_set_cursor(GTK_TREE_VIEW (track_listview), path, NULL, TRUE);
+    gtk_tree_path_free(path);
+
     /* Now actually do the extraction */
     sj_extractor_extract_track (extractor, track, file_path, &error);
     if (error) {
-      GtkWidget *dialog;
-
-      text = g_strdup_printf (_("Sound Juicer could not extract this CD.\nReason: %s"), error->message);
-
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), 0,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       text);
-      g_free (text);
-
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      g_error_free (error);
-      cleanup ();
-      return;
+      goto error;
     }
+    goto local_cleanup;
+error:
+    on_error_cb (NULL, error, NULL);
+    g_error_free (error);
+local_cleanup:
     g_free (file_path);
   }
 }
@@ -680,6 +690,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     title_entry = glade_xml_get_widget (glade, "title_entry");
     artist_entry = glade_xml_get_widget (glade, "artist_entry");
     genre_entry = glade_xml_get_widget (glade, "genre_entry");
+    year_entry = glade_xml_get_widget (glade, "year_entry");
     track_listview = glade_xml_get_widget (glade, "track_listview");
     progress_bar = glade_xml_get_widget (glade, "progress_bar");
     status_bar = glade_xml_get_widget (glade, "status_bar");
@@ -697,6 +708,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   /* TODO: find out why GTK+ needs this to work (see #364371) */
   gtk_button_set_label (GTK_BUTTON (extract_button), _("Stop"));
   gtk_button_set_label (GTK_BUTTON (extract_button), GTK_STOCK_STOP);
+  gtk_widget_show (progress_bar);
   
   /* Reset the progress dialog */
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0);
@@ -707,6 +719,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   gtk_widget_set_sensitive (title_entry, FALSE);
   gtk_widget_set_sensitive (artist_entry, FALSE);
   gtk_widget_set_sensitive (genre_entry, FALSE);
+  gtk_widget_set_sensitive (year_entry, FALSE);
 
   /* Disable the menuitems in the main menu*/ 
   gtk_widget_set_sensitive (play_menuitem, FALSE);
