@@ -30,14 +30,17 @@
 #include <profiles/gnome-media-profiles.h>
 #include <nautilus-burn-drive.h>
 #include <nautilus-burn-drive-selection.h>
+
+#include "sj-util.h"
 #include "gconf-bridge.h"
 #include "sj-extracting.h"
+#include "sj-prefs.h"
 
 extern GladeXML *glade;
 extern GtkWidget *main_window;
 
 static GtkWidget *audio_profile;
-static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject;
+static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject, *check_open;
 static GtkWidget *path_example_label;
 
 #define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
@@ -181,11 +184,18 @@ static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
   const char* base_uri, *current_uri;
   g_return_if_fail (strcmp (entry->key, GCONF_BASEURI) == 0);
 
-  if (entry->value == NULL) {
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (basepath_fcb), g_get_home_dir());
+  base_uri = NULL;
+  if (entry->value != NULL)
+    base_uri = gconf_value_get_string (entry->value);
+
+  if (base_uri == NULL || base_uri[0] == '\0') {
+    char *dir;
+
+    dir = sj_get_default_music_directory ();
+    gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb), dir);
+    g_free (dir);
   } else {
     g_return_if_fail (entry->value->type == GCONF_VALUE_STRING);
-    base_uri = gconf_value_get_string (entry->value);
     current_uri = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb));
     if (current_uri == NULL || strcmp (current_uri, base_uri) != 0) { 
       gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb), base_uri);
@@ -199,7 +209,7 @@ static void pattern_label_update (void)
   char *file_value, *path_value, *example, *format;
   GMAudioProfile *profile;
 
-  static AlbumDetails sample_album = {
+  static const AlbumDetails sample_album = {
     "Help!", /* title */
     "The Beatles", /* artist */
     "Beatles, The", /* sortname */
@@ -210,8 +220,8 @@ static void pattern_label_update (void)
     NULL, /* album ID */
     NULL /* artist ID */
   };
-  static TrackDetails sample_track = {
-    &sample_album,  /*album */
+  static const TrackDetails sample_track = {
+    (AlbumDetails*)&sample_album,  /*album */
     7, /* track number */
     "Ticket To Ride", /* title */
     "The Beatles", /* artist */
@@ -219,7 +229,6 @@ static void pattern_label_update (void)
     0, /* duration */
     NULL, /* track ID */
     NULL, /* artist ID */
-    {0,0,0,0}, /* treeview iterator */
   };
   
   g_object_get (extractor, "profile", &profile, NULL);
@@ -363,7 +372,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
     prefs_dialog = glade_xml_get_widget (glade, "prefs_dialog");
     g_assert (prefs_dialog != NULL);
-    g_object_add_weak_pointer (G_OBJECT (prefs_dialog), (gpointer*)&prefs_dialog);
+    g_object_add_weak_pointer (G_OBJECT (prefs_dialog), (gpointer)&prefs_dialog);
 
     gtk_window_set_transient_for (GTK_WINDOW (prefs_dialog), GTK_WINDOW (main_window));
 
@@ -381,11 +390,13 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
     cd_option = glade_xml_get_widget (glade, "cd_option");
     basepath_fcb = glade_xml_get_widget (glade, "path_chooser");
+    sj_add_default_dirs (GTK_FILE_CHOOSER (basepath_fcb));
     path_option = glade_xml_get_widget (glade, "path_option");
     file_option = glade_xml_get_widget (glade, "file_option");
     audio_profile = glade_xml_get_widget (glade, "audio_profile");
     check_strip = glade_xml_get_widget (glade, "check_strip");
     check_eject = glade_xml_get_widget (glade, "check_eject");
+    check_open = glade_xml_get_widget (glade, "check_open");
     path_example_label = glade_xml_get_widget (glade, "path_example_label");
 
     populate_pattern_combo (GTK_COMBO_BOX (path_option), path_patterns);
@@ -395,6 +406,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
     /* Connect to GConf to update the UI */
     gconf_bridge_bind_property (bridge, GCONF_EJECT, G_OBJECT (check_eject), "active");
+    gconf_bridge_bind_property (bridge, GCONF_OPEN, G_OBJECT (check_open), "active");
     gconf_bridge_bind_property (bridge, GCONF_STRIP, G_OBJECT (check_strip), "active");
     gconf_bridge_bind_property (bridge, GCONF_DEVICE, G_OBJECT (cd_option), "device");
     gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
@@ -415,3 +427,4 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
     gtk_widget_show_all (prefs_dialog);
   }
 }
+
